# -*- encoding: utf-8 -*-
##
## Copyright © 2012 Thibaut Girka <thib@sitedethib.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

from struct import Struct


class Level(object):
    HEADER = Struct('< 32s 32s H')
    POLYGON = Struct('< H H')
    POINT = Struct('< H H')

    def __init__(self, background_path, parallax_background_path=None, polygons=None):
        self.background_path = background_path
        self.parallax_background_path = parallax_background_path
        self.polygons = polygons or []


    @classmethod
    def read(cls, file):
        path1, path2, nb_polygons = cls.HEADER.unpack(file.read(cls.HEADER.size))
        path1 = path1.rstrip(b'\0')
        path2 = path2.rstrip(b'\0')
        polygons = []
        for i in xrange(nb_polygons):
            polygon_type, nb_points = cls.POLYGON.unpack(file.read(cls.POLYGON.size))
            points = [list(cls.POINT.unpack(file.read(cls.POINT.size)))
                      for j in xrange(nb_points)]
            polygons.append((polygon_type, points))

        return cls(path1, path2 or None, polygons)


    def write(self, file):
        file.write(self.HEADER.pack(self.background_path,
                                    self.parallax_background_path or '',
                                    len(self.polygons)))
        for polygon_type, points in self.polygons:
            file.write(self.POLYGON.pack(polygon_type, len(points)))
            for x, y in points:
                file.write(self.POINT.pack(x, y))

