# -*- encoding: utf-8 -*-
##
## Copyright © 2012 Thibaut Girka <thib@sitedethib.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

from itertools import chain

from ptis.game.collidable import Collidable, Polygon



class Obstacle(Collidable):
    def __init__(self, polygon):
        self.type, points = polygon
        Collidable.__init__(self, Polygon(points), (0, 0))


    def on_collide(self, collision):
        pass



class Game(object):
    def __init__(self, level, player):
        self.level = level
        self.player = player
        self.obstacles = [Obstacle(polygon) for polygon in level.polygons]
        self.bullets = [] #TODO
        self.entities = [player]


    def regen(self, level):
        self.level = level
        self.obstacles = [Obstacle(polygon) for polygon in level.polygons]


    def update(self, time):
        objects = self.obstacles# + self.entities + self.bullets
        future_collisions = set([(time, (None, None), (0, 0))])
        to_check = set(self.entities)

        for entity in set(chain(objects, to_check)):
            entity.on_ground = False
            entity.update_bounding_box(time)

        while time > 0:
            future_collisions.update(entity.check_upcoming_collisions(objects)
                                        for entity in to_check)
            next_collision = min(future_collisions)
            ellapsed_time, (entity1, entity2), (nx, ny) = next_collision
            future_collisions.remove(next_collision)
            future_collisions = set((a - ellapsed_time, b, c) for a, b, c in future_collisions)

            time -= ellapsed_time

            for entity in self.entities:
                entity.update(ellapsed_time)

            to_check = set([entity1, entity2]).intersection(self.entities)
            if entity1 is not None:
                entity1.on_collide(next_collision)
                next_collision = ellapsed_time, (entity2, entity1), (-nx, -ny)
                entity2.on_collide(next_collision)

            for entity in self.entities:
                entity.update_bounding_box(time)

        self.bullets = [bullet for bullet in self.bullets if not bullet.removed]
        self.entities = [entity for entity in self.entities if not entity.removed]

