import pygame
import os

from ptis.formats.level import Level


class Loader(object):
    def __init__(self):
        self.level_cache = {}
        self.image_cache = {}


    def load_level(self, level_name):
        if not level_name in self.level_cache:
            with open(os.path.join('data', '%s.map' % level_name), 'rb') as file:
                level = Level.read(file)
            self.level_cache[level_name] = level
        return self.level_cache[level_name]


    def load_image(self, path):
        if not path in self.image_cache:
            image = pygame.image.load(os.path.join('data', path))
            if image.get_alpha() is None:
                image = image.convert()
            else:
                image = image.convert_alpha()
            self.image_cache[path] = image
        return self.image_cache[path]


    def load_level_backgrounds(self, level):
        backgrounds = []
        for path in (level.background_path, level.parallax_background_path):
            backgrounds.append(self.load_image(path) if path else None)
        return backgrounds


    def preload(self, level_names):
        for level in level_names:
            self.load_level_backgrounds(self.load_level(level))

